"""Namedentities workhorse for Python 2."""

from htmlentitydefs import codepoint2name, name2codepoint
import re
import codecs

def unescape(text):
    """Convert from HTML entities (named or numeric) to Unicode characters."""
    
    def fixup(m):
        """Given an HTML entity (named or numeric), return its Unicode
        equivalent. Does not, however, unescape &lt; &gt; and &amp; (decimal 60,
        62, and 38). Those are 'special' in that they are often escaped for very
        important, specific reasons (e.g. to describe HTML within HTML). Any
        messing with them is likely to break things badly."""
        
        text = m.group(0)
        if text[:2] == "&#":            # numeric entity
            try:
                codepoint = int(text[3:-1], 16) if text[:3] == "&#x" else int(text[2:-1])
                if codepoint != 38 and codepoint != 60 and codepoint != 62:
                    return unichr(codepoint)
            except ValueError:
                pass
        else:                           # named entity
            try:
                codepoint = name2codepoint[text[1:-1]]
                if codepoint != 38 and codepoint != 60 and codepoint != 62:
                    return unichr(codepoint)
            except KeyError:
                pass
        return text # leave as is
    return re.sub("&#?\w+;", fixup, text)
    
    
def named_entities_codec(text):
    """Encode codec that converts Unicode characters into named entities (where
    the names are known), or failing that, numerical entities."""
    
    if isinstance(text, (UnicodeEncodeError, UnicodeTranslateError)):
        s = []
        for c in text.object[text.start:text.end]:
            if ord(c) in codepoint2name:
                s.append(u'&%s;' % codepoint2name[ord(c)])
            else:
                s.append(u'&#%s;' % ord(c))
        return ''.join(s), text.end
    else:
        raise TypeError("Can't handle %s" % text.__name__)


codecs.register_error('named_entities', named_entities_codec)
    

def named_entities(text):
    """Given a string, convert its numerical HTML entities to named HTML
    entities. Works by converting the entire string to Unicode characters, then
    re-encoding Unicode characters into named entities (where the names are
    known), or failing that, numerical entities."""
    
    unescaped_text = unescape(text)
    return unescaped_text.encode('ascii', 'named_entities')
    
    
def encode_ampersands(text):
    """Encode ampersands into &amp;"""
    
    text = re.sub('&(?!([a-zA-Z0-9]+|#[0-9]+|#x[0-9a-fA-F]+);)', '&amp;', text)
    return text




#! /usr/bin/env python

from setuptools import setup, find_packages

readme = open('README.txt', 'r')
README_TEXT = readme.read()
readme.close()

setup(
    name='namedentities',
    version='1.0.7',
    author='Jonathan Eunice',
    author_email='jonathan.eunice@gmail.com',
    description='Simple way to convert numeric HTML entities to far more readable named entities.',
    long_description=README_TEXT,
    url='http://bitbucket.org/jeunice/namedentities',
    packages = find_packages(),
    install_requires=[],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: BSD License',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Text Processing :: Filters',
        'Topic :: Text Processing :: Markup :: HTML'
    ]
)
